/'
 ' Project Publisher - Installer Creator Addin for FbEdit and RadASM
 '
 ' Copyright (C) 2010 Arnel A. Borja (Galeon)
 '
 ' This program is free software: you can redistribute it and/or modify it under 
 ' the terms of the GNU General Public License as published by the Free Software 
 ' Foundation, either version 3 of the License, or (at your option) any later 
 ' version.
 '
 ' This program is distributed in the hope that it will be useful, but WITHOUT 
 ' ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 ' FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more 
 ' details.
 '
 ' You should have received a copy of the GNU General Public License along with 
 ' this program.  If not, see <http://www.gnu.org/licenses/>.
 '
 '/

#If Not Defined( FBE_ADDIN ) And Not Defined( RA_ADDIN )
#Define FBE_ADDIN
#EndIf

#Include Once "windows.bi"
#Include Once "win/commctrl.bi"
#Include Once "win/commdlg.bi"
#Include Once "win/shellapi.bi"

#Include Once "FbEdit.bi"
#Include Once "RadASM.bi"

#Include Once "ProjPub.bi"
#Include Once "Settings.bi"
#Include Once "Welcome.bi"
#Include Once "ProjType.bi"
#Include Once "ProjInfo.bi"
#Include Once "ProjWeb.bi"
#Include Once "License.bi"
#Include Once "MultiUser.bi"
#Include Once "InstallFolder.bi"
#Include Once "Files.bi"
#Include Once "Mui.bi"
#Include Once "RemDrvInst.bi"
#Include Once "Shortcuts.bi"
#Include Once "LangDll.bi"
#Include Once "FileAssoc.bi"
#Include Once "Uninst.bi"
#Include Once "Preview.bi"
#Include Once "Publish.bi"
#Include Once "Finish.bi"
#Include Once "Failed.bi"
#Include Once "Resource.bi"

#Ifdef FBE_ADDIN

' For FbEdit
' Returns info on what messages the addin hooks into (in an ADDINHOOKS type).
Function InstallDll Cdecl Alias "InstallDll" ( _
	ByVal hWin As HWND, _
	ByVal hInst As HINSTANCE _
	) As FBEADDINHOOKS Ptr Export

	' The dll's instance
	hInstance = hInst

	' Get pointer to ADDINHANDLES, ADDINDATA and ADDINFUNCTIONS
	lpFbEHandles = Cast( FBEADDINHANDLES Ptr, SendMessage( hWin, FBEAIM_GETHANDLES, 0,0 ) )
	lpFbEData = Cast( FBEADDINDATA Ptr, SendMessage( hWin, FBEAIM_GETDATA, 0, 0 ) )
	lpFbEFunctions = Cast( FBEADDINFUNCTIONS Ptr, SendMessage( hWin, FBEAIM_GETFUNCTIONS, 0, 0 ) )

	' Get IDE path and IDE Ini path
	CurIDE = IDE_FBEDIT
	IDEPath = lpFbEData->AppPath

	'' Fill a MENUITEMINFO structure 
	Dim mii As MENUITEMINFO
	mii.cbSize = SizeOf( MENUITEMINFO )
	mii.fMask  = MIIM_SUBMENU

	'' Get handle to 'Make' popup
	GetMenuItemInfo( lpFbEHandles->hmenu, IDM_FBEMAKE, FALSE, @mii )
	MakeSubMenu = mii.hSubMenu

	'' First get a menu id
	IDM_PUBLISH = SendMessage( hWin, FBEAIM_GETMENUID, 0, 0 )
	'' Add our menu item to Tools menu
	AppendMenu( MakeSubMenu, MF_STRING Or MF_GRAYED, IDM_PUBLISH, GetTranslateString( "10000", "Publish" ) & !"\tCtrl+Alt+F5" )
	AddAccelerator( FVIRTKEY Or FNOINVERT Or FCONTROL Or FALT, VK_F5, IDM_PUBLISH )

	'' Get handle to 'Options' popup
	GetMenuItemInfo( lpFbEHandles->hmenu, IDM_FBEOPTIONS, FALSE, @mii )
	
	'' Get another menu id
	IDM_PUBLISHOPTIONS = SendMessage( hWin, FBEAIM_GETMENUID, 0, 0 )
	'' Add our menu item to Tools menu
	AppendMenu( mii.hSubMenu, MF_STRING, IDM_PUBLISHOPTIONS, GetTranslateString( "10001", "Project Publisher" ) )

	' Load settings
	LoadSettings( @lpFbEData->IniFile )

	' Load license files
	LoadLicenses( IDEPath + "\ProjPub\Licenses" )
	LoadGraphics( IDEPath + "\ProjPub\Graphics" )

	' Messages this addin will hook into
	FbEHooks.hook1 = FBEAIH_COMMAND Or FBEAIH_CLOSE Or FBEAIH_MENUENABLE Or FBEAIH_PROJECTOPEN Or FBEAIH_PROJECTCLOSE
	FbEHooks.hook2 = NULL
	FbEHooks.hook3 = NULL
	FbEHooks.hook4 = NULL

	Return @FbEHooks

End Function

' FbEdit calls this function for every addin message that this addin is hooked into.
' Returning TRUE will prevent FbEdit and other addins from processing the message.
Function DllFunction Cdecl Alias "DllFunction" ( _
	ByVal hWin As HWND, _
	ByVal uMsg As UINT, _
	ByVal wParam As WPARAM, _
	ByVal lParam As LPARAM _
	) As BOOL Export

	Select Case uMsg
		Case FBEAIM_COMMAND
			Select Case LoWord( wParam )
				Case IDM_PUBLISH
					ShowProjectPublishWizard( hWin )
					'
				Case IDM_PUBLISHOPTIONS
					DialogBox( hInstance, MAKEINTRESOURCE( IDD_DLGSETTINGS ), hWin, @SettingsDialogProc )
					'
			End Select
			'
		Case FBEAIM_PROJECTOPEN
			CurProjectFile = lpFbEData->ProjectFile
			CurProjectPath = lpFbEData->ProjectPath
			LoadProjectSettings( @CurProjectFile )
			'
		Case FBEAIM_PROJECTCLOSE
			SaveProjectSettings( @CurProjectFile )
			RemoveAllFolders( @ProjectSettings.Folders )
			RemoveAllFileTypes( @ProjectSettings.FileAssociations )
			CurProjectFile = ""
			CurProjectPath = ""
			'
		Case FBEAIM_MENUENABLE
			Dim uEnable As Integer = MF_BYCOMMAND
			If CurProjectFile <> "" Then
				uEnable Or= MF_ENABLED
			Else
				uEnable Or= MF_GRAYED
			End If
			EnableMenuItem( MakeSubMenu, IDM_PUBLISH, uEnable )
			'
		Case FBEAIM_CLOSE
			DeleteMenu( lpFbEHandles->hMenu, IDM_PUBLISH, MF_BYCOMMAND )
			DeleteMenu( lpFbEHandles->hMenu, IDM_PUBLISHOPTIONS, MF_BYCOMMAND )
			If WelcomeMsgFont Then DeleteObject( WelcomeMsgFont )
			If HeaderTitleFont Then DeleteObject( HeaderTitleFont )
			FreeLicenses
			FreeGraphics
			SaveSettings( lpFbEData->IniFile )
			'
	End Select

	Return FALSE

End Function

#EndIf

#Ifdef RA_ADDIN

' For RadASM
' Returns info on what messages the addin hooks into (in an ADDINHOOKS structure).
Function InstallDllEx CDECL alias "InstallDllEx" ( _
	ByVal hWin as HWND, _
	ByVal fOptions as Long, _
	ByVal hInst as HINSTANCE _
	) As RAADDINHOOKS Ptr Export

	' The dll's instance
	hInstance = hInst

	' Get pointer to ADDINHANDLES, ADDINDATA and ADDINPROCS
	lpRaHandles = Cast( RAADDINHANDLES Ptr, SendMessage( hWin, RAAIM_GETHANDLES, 0,0 ) )
	lpRaData = Cast( RAADDINDATA Ptr, SendMessage( hWin, RAAIM_GETDATA, 0, 0 ) )
	lpRaFunctions = Cast( RAADDINPROCS Ptr, SendMessage( hWin, RAAIM_GETPROCS, 0, 0 ) )

	' Get IDE path and IDE Ini path (FbEdit or RadASM)
	CurIDE = IDE_RADASM
	IDEPath = *lpRaData->lpApp

	'' Fill a MENUITEMINFO structure 
	Dim mii As MENUITEMINFO
	mii.cbSize = SizeOf( MENUITEMINFO )
	mii.fMask  = MIIM_SUBMENU

	'' Get handle to 'Make' popup
	GetMenuItemInfo( lpRaHandles->hmenu, IDM_RAMAKE, FALSE, @mii )
	MakeSubMenu = mii.hSubMenu

	'' First get a menu id
	IDM_PUBLISH = SendMessage( hWin, RAAIM_GETMENUID, 0, 0 )
	'' Add our menu item to Tools menu
	AddAccelerator( FVIRTKEY Or FNOINVERT Or FCONTROL Or FALT, VK_F5, IDM_PUBLISH )
	
	'' Get handle to 'Options' popup
	GetMenuItemInfo( lpRaHandles->hmenu, IDM_RAOPTIONS, FALSE, @mii )
	
	'' Get another menu id
	IDM_PUBLISHOPTIONS = SendMessage( hWin, RAAIM_GETMENUID, 0, 0 )
	'' Add our menu item to Tools menu
	AppendMenu( mii.hSubMenu, MF_STRING, IDM_PUBLISHOPTIONS, GetTranslateString( "10001", "Project Publisher" ) )

	' Load settings
	LoadSettings( lpRaData->lpIniFile )

	' Load license files
	LoadLicenses( IDEPath + "\ProjPub\Licenses" )
	LoadGraphics( IDEPath + "\ProjPub\Graphics" )

	' Messages this addin will hook into
	RaHooks.fHook1 = RAAIH_COMMAND Or RAAIH_CLOSE Or RAAIH_PROJECTOPENED Or RAAIH_PROJECTCLOSE
	RaHooks.fHook2 = NULL
	RaHooks.fHook3 = NULL

	Return @RaHooks

End Function

' RadASM calls this function for every addin message that this addin is hooked into.
Function DllProc Cdecl Alias "DllProc" ( _
	ByVal hWin As HWND, _
	ByVal uMsg As UINT, _
	ByVal wParam As WPARAM, _
	ByVal lParam As LPARAM _
	) As BOOL Export

	Select Case uMsg
		Case RAAIM_COMMAND
			Select Case LoWord( wParam )
				Case IDM_PUBLISH
					ShowProjectPublishWizard( hWin )
					'
				Case IDM_PUBLISHOPTIONS
					DialogBox( hInstance, MAKEINTRESOURCE( IDD_DLGSETTINGS ), hWin, @SettingsDialogProc )
					'
			End Select
			'
		Case RAAIM_PROJECTOPENED
			CurProjectFile = *lpRaData->lpProject
			CurProjectPath = *lpRaData->lpProjectPath
			' Remove ending '\'
			CurProjectPath = Left( CurProjectPath, Len( CurProjectPath ) - 1 )
			LoadProjectSettings( @CurProjectFile )
			AppendMenu( MakeSubMenu, MF_STRING Or MF_ENABLED, IDM_PUBLISH, GetTranslateString( "10000", "Publish" ) & !"\tCtrl+Alt+F5" )
			'
		Case RAAIM_PROJECTCLOSE
			SaveProjectSettings( @CurProjectFile )
			RemoveAllFolders( @ProjectSettings.Folders )
			RemoveAllFileTypes( @ProjectSettings.FileAssociations )
			CurProjectFile = ""
			CurProjectPath = ""
			'
		Case RAAIM_CLOSE
			DeleteMenu( lpRaHandles->hMenu, IDM_PUBLISH, MF_BYCOMMAND )
			DeleteMenu( lpRaHandles->hMenu, IDM_PUBLISHOPTIONS, MF_BYCOMMAND )
			If WelcomeMsgFont Then DeleteObject( WelcomeMsgFont )
			If HeaderTitleFont Then DeleteObject( HeaderTitleFont )
			FreeLicenses
			FreeGraphics
			SaveSettings( *lpRaData->lpIniFile )
			'
	End Select

	Return FALSE

End Function

#EndIf

' This will show the Project Publish Wizard
Sub ShowProjectPublishWizard ( _
	ByVal hwndOwner As HWND _
	)

   Dim As PROPSHEETPAGE psp(0 To PROJPUB_WIZPAGES_END)
   Dim As PROPSHEETHEADER psh

   With psp(PROJPUB_WIZPAGE_WELCOME)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGWELCOME)
		.pfnDlgProc = @WelcomeDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_PROJTYPE)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGPROJTYPE)
		.pfnDlgProc = @ProjTypeDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_PROJINFO)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGPROJINFO)
		.pfnDlgProc = @ProjInfoDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_PROJWEB)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGPROJWEB)
		.pfnDlgProc = @ProjWebDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_LICENSE)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGLICENSE)
		.pfnDlgProc = @LicenseDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_MULTIUSER)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGMULTIUSER)
		.pfnDlgProc = @MultiUserDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_INSTALLFOLDER)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGINSTFOLDER)
		.pfnDlgProc = @InstallFolderDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_FILES)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGFILES)
		.pfnDlgProc = @FilesDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_MUI)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGMUI)
		.pfnDlgProc = @MuiDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_REMDRVINST)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGREMDRVINST)
		.pfnDlgProc = @RemDrvInstDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_SHORTCUTS)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGSHORTCUTS)
		.pfnDlgProc = @ShortcutsDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_LANGDLL)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGLANGDLL)
		.pfnDlgProc = @LangDllDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_FILEASSOC)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGFILEASSOC)
		.pfnDlgProc = @FileAssocDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_UNINST)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGUNINST)
		.pfnDlgProc = @UninstDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_PREVIEW)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGPREVIEW)
		.pfnDlgProc = @PreviewDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_PUBLISH)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGPUBLISH)
		.pfnDlgProc = @PublishDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_FINISH)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGFINISH)
		.pfnDlgProc = @FinishDialogProc
   End With

   With psp(PROJPUB_WIZPAGE_FAILED)
		.dwSize = SizeOf( PROPSHEETPAGE )
		.hInstance = hInstance
		.pszTemplate = MAKEINTRESOURCE(IDD_DLGFAILED)
		.pfnDlgProc = @FailedDialogProc
   End With

	' Wizard caption
	buffer = GetTranslateString( "6000", "Project Publish Wizard" )

	psh.dwSize = SizeOf( PROPSHEETHEADER )
	psh.dwFlags = PSH_USEICONID Or PSH_PROPSHEETPAGE Or PSH_WIZARD Or PSH_USECALLBACK
	psh.hwndParent = hwndOwner
	psh.hInstance = hInstance
	psh.pszIcon = MAKEINTRESOURCE(IDI_PROJPUB)
	psh.pszCaption = @buffer
	psh.nPages = PROJPUB_WIZPAGES
	psh.nStartPage = 0
	psh.ppsp = Cast( LPCPROPSHEETPAGE, @psp(0) )
	psh.pfnCallback = @ProjectPublishWizardCallback

	PropertySheet( @psh )

End Sub

Function ProjectPublishWizardCallback ( _
	ByVal hwndDlg As HWND, _
	ByVal uMsg As UINT, _
	ByVal lParam As LPARAM _
	) As Integer

	Select Case uMsg
		Case PSCB_PRECREATE
			Cast( DLGTEMPLATE Ptr, lParam )->style Or= DS_CONTEXTHELP
			'
	End Select

	Return 0

End Function

Sub DisableUnneededFeatures( _
	)

	DisabledFeatures = NULL
	Select Case ProjectSettings.ProjectType
		Case PROJPUB_PROJTYPE_BASIC
			DisabledFeatures Or= _
				PROJPUB_FEATURES_REMDRVINST Or _
				PROJPUB_FEATURES_FILEASSOC
			'
		Case PROJPUB_PROJTYPE_GAME
			DisabledFeatures Or= _
				PROJPUB_FEATURES_REMDRVINST Or _
				PROJPUB_FEATURES_FILEASSOC
			'
		Case PROJPUB_PROJTYPE_FBEADDIN
			DisabledFeatures Or=  _
				PROJPUB_FEATURES_SHORTCUTS Or  _
				PROJPUB_FEATURES_FILEASSOC Or _
				PROJPUB_FEATURES_RUNONFINISH
			'
		Case PROJPUB_PROJTYPE_ADVANCE
			'
		Case PROJPUB_PROJTYPE_GTKP
			'
		Case PROJPUB_PROJTYPE_APPPLUGIN
			DisabledFeatures Or=  _
				PROJPUB_FEATURES_REMDRVINST Or  _
				PROJPUB_FEATURES_SHORTCUTS Or  _
				PROJPUB_FEATURES_FILEASSOC Or _
				PROJPUB_FEATURES_RUNONFINISH
			'
	End Select

End Sub

'' Get translation of strings.
Function GetTranslateString( _
	ByRef KeyName As Const String, _
	ByRef KeyValueDefault As Const String _
	) As String

	If CurIDE = IDE_FBEDIT Then
		'' Get translation
		Dim KeyValue As String = lpFbEFunctions->FindString( _
			lpFbEData->hLangMem, _
			ADDIN_NAME, _
			KeyName _
			)
	
		'' Check if valid
		If KeyValue <> "" Then
			Return KeyValue
		Else
			Return KeyValueDefault
		End If
	Else
		Return KeyValueDefault
	End If

End Function

Sub AddAccelerator( _
	ByVal fvirt As Integer, _
	ByVal akey As Integer, _
	ByVal id As Integer _
	)

	Dim Found As Integer
	Dim acl(500) As ACCEL
	Dim nAccel As Integer
	If CurIDE = IDE_FBEDIT Then
		nAccel = CopyAcceleratorTable( lpFbEHandles->haccel, NULL, 0 )
		CopyAcceleratorTable( lpFbEHandles->haccel, @acl(0), nAccel )
		DestroyAcceleratorTable( lpFbEHandles->haccel )
	Else
		nAccel = CopyAcceleratorTable( lpRaHandles->haccel, NULL, 0 )
		CopyAcceleratorTable( lpRaHandles->haccel, @acl(0), nAccel )
		DestroyAcceleratorTable( lpRaHandles->haccel )
	End If

	' Check if id or accelerator exist
	For i As Integer = 0 To nAccel-1
		If acl(i).cmd = id Then
			' id exist, update accelerator
			acl(i).fVirt = fvirt
			acl(i).key = akey
			Found = -1
			Exit For
		ElseIf acl(i).fVirt = fvirt And _
			acl(i).key = akey Then
			' Accelerator exist, update id
			acl(i).cmd = id
			Found = -1
			Exit For
		End If
	Next

	If Not Found Then
		' Add new accelerator
		acl(nAccel).fVirt = fvirt
		acl(nAccel).key = akey
		acl(nAccel).cmd = id
		nAccel += 1
	End If

	If CurIDE = IDE_FBEDIT Then
		lpFbEHandles->haccel = CreateAcceleratorTable( @acl(0), nAccel )
	Else
		lpRaHandles->haccel = CreateAcceleratorTable( @acl(0), nAccel )
	End If

End Sub

Sub ErrorBox( _
	ByVal hWin As HWND, _
	ByRef text As Const String _
	)

	MessageBox( hWin, text, NULL, MB_OK Or MB_ICONERROR )

End Sub


Function OpenFileName( _
	ByVal hWin As HWND, _
	ByVal lpszTitle As ZString Ptr, _
	ByVal lpszInitialDir As ZString Ptr, _
	ByVal lpszFilter As ZString Ptr _
	) As String

	Dim ofn As OPENFILENAME
	Dim buff As ZString * MAX_PATH+1

	With ofn
		.lStructSize = SizeOf(OPENFILENAME)
		.hwndOwner = hWin
		.hInstance = hInstance
		.lpstrInitialDir = lpszInitialDir
		.lpstrFile = @buff
		.nMaxFile = MAX_PATH
		.lpstrFilter = lpszFilter
		.lpstrTitle = lpszTitle
		.Flags = OFN_PATHMUSTEXIST Or OFN_HIDEREADONLY Or OFN_EXPLORER
	End With

	GetOpenFileName( @ofn )

	Return buff

End Function

Function GetDir( _
	ByRef Path As Const String _
	) As String

	Return Left( Path, InStrRev( Path, "\" ) - 1 )

End Function

Function GetFileName( _
	ByRef Path As Const String _
	) As String

	Return Mid( Path, InStrRev( Path, "\" ) + 1 )

End Function

Function GetBaseName( _
	ByRef Path As Const String _
	) As String

	Dim FileName As String = GetFileName( Path )

	Return Left( FileName, InStrRev( FileName, "." ) - 1 )

End Function

Function GetUpFromParentDir( _
	ByRef Path As Const String _
	) As Integer

	Dim TempPath As String = Path

	Dim BackslashPos As Integer = InStr( TempPath, "\" )
	Dim BackslashNum As Integer
	
	Do While BackslashPos
		TempPath = Mid( TempPath, BackslashPos + 1 )
		BackslashNum += 1
		BackslashPos = InStr( TempPath, "\" )
	Loop
	
	Return BackslashNum

End Function

Function EnableDlgItem( _
	ByVal hWin As HWND, _
	ByVal nIDDlgItem As Integer, _
	ByVal bEnable As BOOL _
	) As BOOL

	Return EnableWindow( GetDlgItem( hWin, nIDDlgItem ), bEnable ) 

End Function

Function GetRelativePath( _
	ByVal AbsolutePath As Const String, _
	ByVal Folder As Const String _
	) As String

	Dim PrependPath As String
	Dim TempFolder As String = Folder

	Do
		If UCase( Left( AbsolutePath, Len( TempFolder ) + 1 ) ) = UCase( TempFolder & "\" ) Then
			Return PrependPath & Mid( AbsolutePath, Len( TempFolder ) + 2 )
		Else
			PrependPath &= "..\"
			TempFolder = Left( TempFolder, InStrRev( TempFolder, "\" ) - 1 )
			' Check if string is empty or equal to drive (i.e. "C:")
			If TempFolder = "" Or TempFolder = Left( Folder, 2 ) Then
				Return AbsolutePath
			End If
		End If
	Loop

End Function

Function Replace( _
	ByRef InputString As Const String, _
	ByRef SearchString As Const String, _
	ByRef Replacement As Const String _
	) As String

	Dim As Integer x, y = 1
	Dim Temp As String = InputString
	Dim RetStr As String

	Do While y
		x = InStr( Temp, SearchString )
		If x Then
			Print SearchString & " is found at " & x
			RetStr &= Left( Temp, x-1 ) + Replacement
			Print "RetStr is now " & RetStr
			Temp = Mid( Temp, x + Len( SearchString ) )
			Print
			Continue Do
		End If
		y = 0
	Loop

	If Temp <> "" And RetStr <> "" Then
		Return RetStr & Temp
	ElseIf RetStr <> "" Then
		Return RetStr
	Else
		Return InputString
	End If

End Function

Function EscapeNSIS( _
	ByRef UnescapedStr As Const String _
	) As String
	
	Dim As Integer DollarSignPos, DoubleQuotePos
	Dim y As Integer = 1
	Dim TempStr As String = UnescapedStr
	Dim RetString As String

	Print "UnescapedStr is " & UnescapedStr

	'TempStr = Replace( TempStr, "$", "$\$" )
	TempStr = Replace( TempStr, """", "$\""" )

	Return TempStr

End Function

Function FilterNSISIdentifier( _
	ByRef NSISIdentifier As Const String _
	) As String

	Dim TempString As String = NSISIdentifier
	Dim RetString As String
	Dim Char As Unsigned Integer

	Do While TempString <> ""
		Char = Asc( TempString )
		If (Char >= &h41 And Char <= &h5A) Or _	' A-Z
			(Char >= &h61 And Char <= &h7A) Or _	' a-z
			(Char >= &h30 And Char <= &h39) Or _	' 0-9
			Char = &h5F Then _							' _
			RetString &= Chr( Char )
		TempString = Mid( TempString, 2 )
	Loop

	Return RetString

End Function

